package com.bonait.bnframework.business;

import com.bonait.bnframework.HBL.Task;
import com.bonait.bnframework.common.ModbusSlave.ModbusTcpSlave;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.db.res.UserLogEnum;
import com.bonait.bnframework.common.helper.I.IThread;
import com.bonait.bnframework.common.helper.MessageLog;
import com.bonait.bnframework.common.helper.TTrig;
import com.bonait.bnframework.common.helper.ThreadManager;
import com.bonait.bnframework.common.modbus.IOperateResult;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

public abstract class ControlBase {

    public void Init(){
        RobotData.FryingStatus.put(1,new FryingPanData());
        RobotData.FryingStatus.put(2,new FryingPanData());
        RobotData.FryingStatus.put(3,new FryingPanData());
        RobotData.FryingStatus.put(4,new FryingPanData());
        HeartbeatStatusListen();
        ConnectRobot();
        ConnectYJDevice();
        FryingpanModbusHelper.getInstance().Init();
        ThreadManager.Get().StartLong("炒锅控制业务", true, new IThread() {
            @Override
            public void Run() throws InterruptedException {
                Control();
                Thread.sleep(100);
            }

            @Override
            public void RunComplete() throws InterruptedException {

            }
        });
        MbSlave.Init();
    }

    /**
     * 根据炒锅编号解析对应的炒锅控制地址
     * @param num
     * @param offset
     * @return
     */
    protected int GetAddress(int num,int offset)
    {
        return (num-1)*100+offset;
    }

    /**
     * 解析商品ID地址
     * @param num
     * @param offset
     * @return
     */
    protected int GetHoldAdd(int num,int offset){
        return (num-1)*20+offset;
    }

    /**
     * 解析料仓绑定
     * @param num
     * @return
     */
    protected int GetSiloAdd(int num){
        return (num-1)*20+100;
    }

    protected Integer GetRobotAdd(int num,int offset){
        //M500--4596
//        return "M"+ 500+offset+(num-1)*10;
        return 4596+offset+(num-1)*10;
    }

    /**
     * 获取心跳状态
     * @param num
     * @return
     */
    protected boolean GetHeartbeatStatus(int num){
        return HeartbeatStatus[num-1];
    }

    /**
     * 炒锅心跳状态
     */
    protected boolean[] HeartbeatStatus=new boolean[4];

    /**
     * 炒锅心跳时间
     */
    protected long[] StartTime=new long[4];
//    protected long[] EndTime=new long[4];

    /**
     * 强制结束信号
     */
    protected boolean[] ForceEnd =new boolean[4];//强制结束

    /**
     * 需要移除的任务
     */
    protected List<Integer> RemoveId=new ArrayList<Integer>();

    /**
     * 强制结束指定炒锅任务
     * @param num
     */
    public void SetForceEnd(int num){
        if(num>0){
            ForceEnd[num-1]=true;
        }
    }

    /**
     * 复位强制结束信号
     * @param num
     */
    private void resetForceEnd(int num){
        if(num>0){
            ForceEnd[num-1]=false;
        }
    }

    /**
     * 设备心跳超时时间
     */
    protected long TimeOut=5000;
    public LinkedHashMap<Integer,FryingPanPar> FryingPan=new LinkedHashMap<>();
    protected ModbusTcpSlave MbSlave=new ModbusTcpSlave();

    /**
     * 机器人地址解析
     * @param address
     * @return
     */
    public   String RobotAddress(String address){
        if (address == null) return "";
        if (address.length() > 0) {
            address = address.trim();
            if (address.toUpperCase().contains("M") &&address.length() >=2){
                String res = address.substring(1);
                if (res != null) {
                    try {
                        Integer tempAdd = Integer.parseInt(res);
                        if(tempAdd>=0){
                            return (tempAdd+4096)+"";
                        }
                    } catch (NumberFormatException e) {
                        return "";
                    }
                }
            }
        }
        return "";
    }

    protected  void ConnectYJDevice(){
        Task.Run(()->{
            YJDeviceHelper.getInstance().TcpConnect(ConfigName.getInstance().DeviceAddress, ConfigName.getInstance().DevicePort, new IOperateResult() {
                @Override
                public void onSuccess() {
                    YJDeviceHelper.getInstance().ReadData();
                }

                @Override
                public void onFailure(String message) {

                }

                @Override
                public void onFinally() {

                }
            });
        });
    }

    protected void ConnectRobot(){
        new Thread(()->{
            try {
                RobotModbus.getInstance().TcpConnect(ConfigName.getInstance().RobotAddress, ConfigName.getInstance().RobotPort, new IOperateResult() {
                    @Override
                    public void onSuccess() {
                        ConfigName.getInstance().PlcIsConnect = true;
                        ThreadManager.Get().StartLong("机器人数据监听", true, new IThread() {
                            @Override
                            public void Run() throws InterruptedException {
                                RobotDataListen();
                                Thread.sleep(100);
                            }

                            @Override
                            public void RunComplete() throws InterruptedException {

                            }
                        });
                    }

                    @Override
                    public void onFailure(String message) {
                        ConfigName.getInstance().PlcIsConnect = false;
//                        ToastUtils.info("设备 " + ConfigName.getInstance().Address + " 连接失败：" + message);
                    }

                    @Override
                    public void onFinally() {

                    }
                });
            }catch (Exception e  )
            {
                MessageLog.ShowInfo("重新连接Modbus异常，" + e.getMessage());
            }
        }).start();
    }

    /**
     * 商品是否验证成功
     * @param num
     * @return
     */
    public boolean GetGoodVery(int num){
        return MbSlave.ReadBoolean(GetAddress(num,AddressData.GoodDataVery));
    }

    /**
     * 获取指定炒锅的状态
     * @param num
     * @return
     */
    public boolean GetFryingPan(int num){
        return MbSlave.ReadBoolean(GetAddress(num,AddressData.FryingPanHide))&&GetHeartbeatStatus(num);
    }

    /**
     * 商品下发
     */
    public void GoodDown(String id,int num){
        MbSlave.WriteBoolean(GetAddress(num,AddressData.GoodDataVery),false);//复位商品验证
        MbSlave.WriteBoolean(GetAddress(num,AddressData.PutMainMaterial),false);//复位放主料完成
        MbSlave.WriteBoolean(GetAddress(num,AddressData.PutAccessories),false);//复位放辅料完成
        MbSlave.WriteBoolean(GetAddress(num,AddressData.RobotServingSpace),false);//复位机器人到达出餐位
        MbSlave.WriteBoolean(GetAddress(num,AddressData.FryingpanForceEnd),false);//复位强制结束信号
        MbSlave.WriteBoolean(GetAddress(num,AddressData.ForceEnd),false);//复位强制结束控制
        MbSlave.WriteString(id,GetHoldAdd(num,0));//写入商品ID
        MbSlave.WriteBoolean(GetAddress(num,AddressData.GoodDown),true);//写入商品下发
        MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"写入的商品id"+id);
    }

    /**
     * 强制结束信号检测
     * @param num
     * @return
     */
    protected boolean EndProcess(int num){
        if(ForceEnd[num-1]||MbSlave.ReadBoolean(GetAddress(num,AddressData.FryingpanForceEnd))){
            if( ForceEnd[num-1]){
                MbSlave.WriteBoolean(GetAddress(num,AddressData.ForceEnd),true);//强制结束
            }

            long startTime=System.currentTimeMillis();
            while(!MbSlave.ReadBoolean(GetAddress(num,AddressData.FryingpanForceEnd))&&(System.currentTimeMillis()-startTime)<=4000){
                try {
                    Thread.sleep(100);
                }catch (InterruptedException ex){

                }
            }

            if(MbSlave.ReadBoolean(GetAddress(num,AddressData.FryingpanForceEnd))){
                MbSlave.WriteBoolean(GetAddress(num,AddressData.FryingpanForceEnd),false);//复位强制结束信号
            }

            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,num+" 号炒锅取消任务");
            if(FryingPan.containsKey(num))
                RemoveId.add(num);
           return true;
        }
        else return false;
    }

    protected int GetBooleanIndex(boolean[] bools,boolean value){
        if(bools==null)return -1;
        int index=-1;
        for(int i=0;i<bools.length;i++){
            if(bools[i]==value){
                index=i;
                break;
            }
        }
        return index;
    }

    public ICompleteNotify completeNotify;

    public IStatusNotify statusNotify;

    boolean[] First = new boolean[4];

    /**
     * 设备心跳监听
     */
    protected void HeartbeatStatusListen(){
        ThreadManager.Get().StartLong("设备心跳监听", true, new IThread() {
            @Override
            public void Run() throws InterruptedException {

                String[] status=new String[4];
                for (int i=0;i<4;i++){

                    if(StartTime[i]==0&&!First[i]){
                        MbSlave.WriteBoolean(GetAddress(i+1,AddressData.Heartbeat),true);
                        First[i]=true;
                    }

                    if(TTrig.get("心跳检测"+String.valueOf(i)).Start(MbSlave.ReadBoolean(GetAddress(i+1,AddressData.Heartbeat)))){
                        StartTime[i]=System.currentTimeMillis();
                        MbSlave.WriteBoolean(GetAddress(i+1,AddressData.Heartbeat),true);
                    }

                    HeartbeatStatus[i]=StartTime[i]>0&&(System.currentTimeMillis()-StartTime[i])<=TimeOut;

                    if ( HeartbeatStatus[i]&&MbSlave.ReadBoolean(GetAddress(i+1,AddressData.FryingPanHide))){
                        status[i]="空闲中";
                    }
                    else{
                        if(HeartbeatStatus[i]) status[i]="忙碌中";
                        else status[i]="未连接";
                    }
                }

                if(statusNotify!=null)
                    statusNotify.OnStatusChanged(status[0],status[1],status[2],status[3]);
                Thread.sleep(100);
            }

            @Override
            public void RunComplete() throws InterruptedException {

            }
        });
    }

    /**
     * 机器人投主料控制
     * @param IngredientsLocs 主料位置
     * @param FryingPanNum 炒锅位置
     */
    protected void RobotFeedingMainMaterials(short IngredientsLocs,short FryingPanNum){
        try
        {
            if(IngredientsLocs>0){
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人从"+IngredientsLocs+"号主料位，放主料到"+FryingPanNum + "号锅");
                RobotModbus.getInstance().WriteShort("GI0",IngredientsLocs,1,null);//写入机器人主任务,指定主料位置
                RobotModbus.getInstance().WriteShort("GI1",FryingPanNum,1,null);//写入机器人子任务，指定炒锅位置
                RobotModbus.getInstance().WriteBool(GetRobotAdd(FryingPanNum,0).toString(),true,1,null);//写入机器人允许放料信号
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人放菜完成");
                while(!RobotData.FryingStatus.get((int)FryingPanNum).PutMaterialComplete&&!ForceEnd[FryingPanNum-1]){Thread.sleep(100); }//等待放主料完成
                if(EndProcess(FryingPanNum))return;
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人放菜完成");
            }
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.RobotPutIngredientsComplete),true);//机器人放主料完成
        }
        catch(InterruptedException ex){

        }
    }

    /**
     * 机器人投辅料控制
     * @param AccessoriesLoc 辅料位置
     * @param FryingPanNum 炒锅位置
     */
    protected void RobotFeedingAuxiliaryMaterials(short AccessoriesLoc,short FryingPanNum){
        try
        {
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人从" +AccessoriesLoc+"号辅料位，放辅料到"+FryingPanNum + "号锅");
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.FryingPanRequestAccessories),false);//复位投辅料请求信号
            if(AccessoriesLoc-10>0){
                RobotModbus.getInstance().WriteShort("GI0",AccessoriesLoc,1,null);//写入机器人主任务,设置辅料位置
                RobotModbus.getInstance().WriteShort("GI2",FryingPanNum,1,null);//写入机器人子任务，设置炒锅位置
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人倒调料完成");
                while(!RobotData.FryingStatus.get((int)FryingPanNum).FallSeasoningComplete&&!ForceEnd[FryingPanNum-1]){Thread.sleep(100);}//等待机器人倒调料完成
            }
            MbSlave.WriteString(GetHoldAdd(FryingPanNum,0));//复位商品id
            if(EndProcess(FryingPanNum))return;
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人倒调料完成");
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.PutAccessories),true);//设置机器人放辅料完成信号
        }
        catch(InterruptedException ex){

        }
    }

    /**
     * 移除任务检测
     */
    protected void RemoveTask(){
        for(Integer key:RemoveId){
            if(FryingPan.containsKey(key))
            {
                FryingPan.remove(key);
                resetForceEnd(key);
                if (completeNotify!=null)
                    completeNotify.onComplete(key);
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"删除"+key+"号炒锅任务");

            }
        }
        RemoveId.clear();
    }
    /**
     * 机器人出餐控制
     * @param FryingPanNum 炒锅位置
     */
    protected void RobotMealControl(int FryingPanNum){
        try
        {
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人到"+FryingPanNum+"号炒锅位置");
            //等待机器人到达炒锅位置
            while(!RobotData.FryingStatus.get((int)FryingPanNum).RequestDiningOut&&!ForceEnd[FryingPanNum-1]){Thread.sleep(100);}
            if(EndProcess(FryingPanNum))return;
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人到"+FryingPanNum+"号炒锅位置");
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.RobotServingSpace),true);//通知炒锅，机器人到达出餐位

            //等待炒锅到达减速位
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人到"+FryingPanNum+"号炒锅减速位置");
            while (!MbSlave.ReadBoolean(GetAddress(FryingPanNum,AddressData.DecelerationInPlace))&&!ForceEnd[FryingPanNum-1]){Thread.sleep(50);}
            if(EndProcess(FryingPanNum)) return;
            RobotModbus.getInstance().WriteBool(GetRobotAdd(FryingPanNum,1).toString(),true,1,null);//设置机器人炒锅减速到位
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人到达"+FryingPanNum+"号炒锅减速位置");

            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待"+FryingPanNum+"号炒锅出餐完成");
            //等待炒锅出餐完成
            while (!MbSlave.ReadBoolean(GetAddress(FryingPanNum,AddressData.FryingPanDiningOut))&&!ForceEnd[FryingPanNum-1]){Thread.sleep(100);}
            if(EndProcess(FryingPanNum)) return;
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,FryingPanNum+"号炒锅出餐完成");
            YJDeviceHelper.getInstance().ConveyorLineControl(false);//停止输送线

            //设置机器人出餐完成
            RobotModbus.getInstance().WriteBool(RobotAddress("M530"),true,1,null);//允许放盆
            RobotModbus.getInstance().WriteBool(GetRobotAdd(FryingPanNum,2).toString(),true,1,null);
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人出餐完成");

            //等待机器人出餐完成
            while(!RobotData.FryingStatus.get((int)FryingPanNum).DiningOutComplete&&!ForceEnd[FryingPanNum-1]){Thread.sleep(100);}
            if(EndProcess(FryingPanNum)) return;
            if(PeripheralData.ConveyorLineInputBowl){
                YJDeviceHelper.getInstance().ConveyorLineControl(true);//启动输送线
            }
            PeripheralData.PutBlowComplete=true;
            MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人出餐完成");
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.RequestingMeaDelivery),false);;  //复位炒菜完成，出餐请求
            MbSlave.WriteBoolean(GetAddress(FryingPanNum,AddressData.FryingPanDiningOut),false); //复位炒锅出餐完成信号

            //通知界面可以继续下发任务
            if (completeNotify!=null)
            {
                completeNotify.onComplete(FryingPanNum);
                MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"通知界面"+FryingPanNum+"号炒锅执行完成");
            }

            if(FryingPan.containsKey(FryingPanNum))
                RemoveId.add(FryingPanNum);

        }
        catch(InterruptedException ex){

        }
    }

    protected abstract void  RobotDataListen()throws InterruptedException;

    protected abstract void Control()throws InterruptedException;

}
