package com.bonait.bnframework.business;

import com.bonait.bnframework.HBL.Task;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.db.QueryDB;
import com.bonait.bnframework.common.db.mode.BPA_MATERIAL;
import com.bonait.bnframework.common.helper.I.IRun;
import com.bonait.bnframework.common.helper.I.IRunT;
import com.bonait.bnframework.common.model.mode.MouthFacingEnum;

import java.util.concurrent.ConcurrentHashMap;

public class FryingpanModbusHelper {
    private volatile static FryingpanModbusHelper _Instance;
    public synchronized  static FryingpanModbusHelper getInstance(){
        if(_Instance==null)_Instance=new FryingpanModbusHelper();
        return _Instance;
    }

    private FryingpanModbusHelper(){
        RobotData.Fryingpan1.ChangeNotify=(s)->{
            String[] res = s.split("[;]");
            if(devices.containsKey(1)){
                 devices.get(1).MaterialName=res;
            }
            if(res!=null&&res.length>=4){
                AddMaterial(res);
            }
        };

        RobotData.Fryingpan2.ChangeNotify=(s)->{
            String[] res = s.split("[;]");
            if(devices.containsKey(2)){
                devices.get(2).MaterialName=res;
            }
            if(res!=null&&res.length>=4){
                AddMaterial(res);
            }
        };
    }

    private void AddMaterial(String[] res){
        if(res==null)return;
        for (int i=0;i<res.length;i++){
            BPA_MATERIAL material = QueryDB.GetMaterialName(res[i]);
            if(material==null){
                BPA_MATERIAL addMaterial = new BPA_MATERIAL();
                addMaterial.name=res[i];
                addMaterial.type=0;
                QueryDB.AddMaterial(addMaterial);
            }
        }
    }

    private ConcurrentHashMap<Integer,FryingpanModbusBase> devices = new ConcurrentHashMap<>();

    public void Dispose(){
        devices.forEach((key,value)->{value.Close();});
        _Instance=null;
    }

    public void Init(){
        devices.put(1,new FryingpanModbusBase());
        devices.put(2,new FryingpanModbusBase());
        devices.get(1).Name="一号炒锅数据监听";
        devices.get(2).Name="二号炒锅数据监听";
        Task.Run(()->{ devices.get(1).TcpConnect(ConfigName.getInstance().F1Address, ConfigName.getInstance().F1Port,null);});
        Task.Run(()->{ devices.get(2).TcpConnect(ConfigName.getInstance().F2Address, ConfigName.getInstance().F2Port,null);});
    }

    public void getModbus(int num,IRunT<FryingpanModbusBase> modbus){
        if(devices.containsKey(num)&&modbus!=null){
            modbus.Run(devices.get(num));
        }
    }

    public boolean isConnect(int num){
        if(devices.containsKey(num)){
           return devices.get(num).IsConnected;
        }
        return false;
    }

    /**
     * 抽料位反馈
     * @param num
     * @return
     */
    public boolean getMaterialLocFeedback(int num){
        if(devices.containsKey(num)){
            return devices.get(num).MaterialLocFeedback;
        }
        return false;
    }

    /**
     * 获取搅拌反馈
     * @param num
     * @return
     */
    public boolean getStirFeedback(int num){
        if(devices.containsKey(num)){
            return devices.get(num).StirFeedback;
        }
        return false;
    }

    /**
     * 设置制作状态
     * @param num
     */
    public void setMake(int num,boolean isMake){
        if(devices.containsKey(num)){
             devices.get(num).IsMake=isMake;
        }
    }

    /**
     * 获取制作状态
     * @param num
     */
    public boolean getMake(int num){
        if(devices.containsKey(num)){
            return devices.get(num).IsMake;
        }
        return false;
    }

    /**
     * 清洗位反馈
     * @param num
     * @return
     */
    public boolean getClearLocFeedback(int num){
        if(devices.containsKey(num)){
            return devices.get(num).ClearLocFeedback;
        }
        return false;
    }

//    /**
//     * 当前温度更新通知
//     * @param num
//     * @return
//     */
//    public void getCurrentTemperature(int num, IRunT<Integer> action){
//        if(devices.containsKey(num)){
//            devices.get(num).CurrentTemperature.ChangeNotify=(s->{
//                if(action!=null)action.Run(s);
//            });
//        }
//    }

    /**
     * 出料控制
     * @param num
     * @param ce
     * @param value
     */
    public void deviceControl(int num, ControlEnum ce, boolean value){
        if(devices.containsKey(num)){
             devices.get(num).deviceControl(ce,value);
        }
    }

    /**
     * 获取物料名称
     * @param num
     * @return
     */
    public String[] getMaterialName(int num){
        if(devices.containsKey(num)){
            return devices.get(num).MaterialName;
        }
        return null;
    }

    /**
     * 搅拌控制
     * @param num
     * @param frequency
     * @param status
     */
    public void setStir(int num,short frequency,boolean status){
        if(devices.containsKey(num)){
            devices.get(num).setStir(frequency,status);
        }
    }

    /**
     * 火力设置
     * @param num
     * @param value
     */
    public void setFire(int num,short value){
        if(devices.containsKey(num)){
            devices.get(num).setFire(value);
        }
    }

    /**
     * 锅体朝向控制
     * @param num
     * @param mouth
     */
    public void SelfResetControl(int num, MouthFacingEnum mouth ){
        if(devices.containsKey(num)){
            devices.get(num).SelfResetControl( mouth);
        }
    }

    /**
     * 清洗抽水量
     * @param num
     * @param value
     */
    public void setPumpingCapacity(int num,short value){
        if(devices.containsKey(num)){
            devices.get(num).setPumpingCapacity( value);
        }
    }

}
