package com.bonait.bnframework.business;

import android.util.Log;

import com.bonait.bnframework.common.db.res.UserLogEnum;
import com.bonait.bnframework.common.helper.MessageLog;
import com.bonait.bnframework.common.modbus.ModbusMaster;

import java.util.ArrayList;
import java.util.List;

/**
 * 一拖四设备控制
 */
public class RobotControl extends ControlBase {
    //region 私有
    private static RobotControl mInstance; //实例变量设置私有，防止直接通过类名访问

    public static synchronized RobotControl getInstance() { //静态同步方法作为唯一的实例对象获取方式
        if (mInstance == null) {
            mInstance = new RobotControl();
        }
        return mInstance;
    }

    private RobotControl() {

    }

    @Override
    public  void RobotDataListen()throws InterruptedException{
        //机器人急停
        RobotModbus.getInstance().ReadBool("M640",1,1,value->{
            for (int i =0;i<4;i++){
                MbSlave.WriteBoolean(GetAddress(i+1,AddressData.EStop),value[0]);
            }
        });

        //M640--1280--4736
        RobotModbus.getInstance().ReadBool("4736",6,1,value->{
            RobotData.PutEmptyBasinComplete=value;
        });

        //M660--1294--4756
        RobotModbus.getInstance().ReadBool("4756",1,1,value->{
            RobotData.RobotHome=value[0];
        });

        //GI5--5
        RobotModbus.getInstance().ReadShort("5",1,1,value->{
            RobotData.RobotTaskFeedback=value[0];
        });

        //M600--1258-4696
        RobotModbus.getInstance().ReadBool("4696",4,1,value->{
            RobotData.FryingStatus.get(1).PutMaterialComplete=value[0];
            RobotData.FryingStatus.get(1).FallSeasoningComplete=value[1];
            RobotData.FryingStatus.get(1).RequestDiningOut=value[2];
            RobotData.FryingStatus.get(1).DiningOutComplete=value[3];
        });

        //M610--1262-4706
        RobotModbus.getInstance().ReadBool("4706",4,1,value->{
            RobotData.FryingStatus.get(2).PutMaterialComplete=value[0];
            RobotData.FryingStatus.get(2).FallSeasoningComplete=value[1];
            RobotData.FryingStatus.get(2).RequestDiningOut=value[2];
            RobotData.FryingStatus.get(2).DiningOutComplete=value[3];
        });

        //M620--126C-4716
        RobotModbus.getInstance().ReadBool("4716",4,1,value->{
            RobotData.FryingStatus.get(3).PutMaterialComplete=value[0];
            RobotData.FryingStatus.get(3).FallSeasoningComplete=value[1];
            RobotData.FryingStatus.get(3).RequestDiningOut=value[2];
            RobotData.FryingStatus.get(3).DiningOutComplete=value[3];
        });

        //M630--1276-4726
        RobotModbus.getInstance().ReadBool("4726",4,1,value->{
            RobotData.FryingStatus.get(4).PutMaterialComplete=value[0];
            RobotData.FryingStatus.get(4).FallSeasoningComplete=value[1];
            RobotData.FryingStatus.get(4).RequestDiningOut=value[2];
            RobotData.FryingStatus.get(4).DiningOutComplete=value[3];
        });

        RobotData.RobotIdle=RobotData.RobotHome&&RobotData.RobotTaskFeedback==0;//机器人空闲状态
    }

    @Override
    public void Control()throws InterruptedException{
        if (FryingPan.size()>0){
            for(FryingPanPar item:FryingPan.values()){
                //如果没有放主料,并且投料机构OK
                if( RobotData.RobotIdle&&!item.IsPutIngredients && MbSlave.ReadBoolean(GetAddress(item.FryingPanNum,AddressData.FeedingMechanism))){
                    item.IsPutIngredients=true;
                    RobotFeedingMainMaterials((short)item.IngredientsLocs[0],(short)item.FryingPanNum);
                }
                if(EndProcess(item.FryingPanNum)) break;//强制结束检测

                //投辅料请求
                if( RobotData.RobotIdle&& MbSlave.ReadBoolean(GetAddress(item.FryingPanNum,AddressData.FryingPanRequestAccessories))){
                    int tempLoc = item.AccessoriesLocs[item.AddAccessoriesCount]+10;
                    RobotFeedingAuxiliaryMaterials((short)tempLoc,(short) item.FryingPanNum);//机器人投辅料控制
                    item.AddAccessoriesCount++;
                }

                if(EndProcess(item.FryingPanNum)) break;//强制结束检测

                //倒主料完成，放主料盒子到货架请求
                if( RobotData.RobotIdle&&MbSlave.ReadBoolean(GetAddress(item.FryingPanNum,AddressData.FryingPnaMainMaterialComplete)))
                {
                    MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人从" +item.FryingPanNum+"号炒锅取料盆到"+item.IngredientsLocs[0] + "号货架");
                    RobotModbus.getInstance().WriteShort("GI0",(short) (item.FryingPanNum+20),1,null);//写入机器人主任务,指定锅号
                    RobotModbus.getInstance().WriteShort("GI3",(short) item.IngredientsLocs[0],1,null);//写入机器人子任务，指定货架位置
                    MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"等待机器人放空盆完成");
                    //等待放空盆完成
                    while(!RobotData.PutEmptyBasinComplete[item.IngredientsLocs[0]-1]&&!ForceEnd[item.FryingPanNum-1]) { Thread.sleep(100);}
                    if(EndProcess(item.FryingPanNum)) break;//强制结束检测
                    MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 , "机器人放空盆完成");
                    MbSlave.WriteBoolean(GetAddress(item.FryingPanNum,AddressData.FryingPnaMainMaterialComplete),false);//复位倒主料完成信号
                }
                if(EndProcess(item.FryingPanNum)) break;//强制结束检测

                //炒菜完成请求出餐
                if( RobotData.RobotIdle&&MbSlave.ReadBoolean(GetAddress(item.FryingPanNum,AddressData.RequestingMeaDelivery))){
                    //验证输送线进料位置无菜盆
                    boolean ConveyorLineOutputBowl = PeripheralData.ConveyorLineOutputBowl.getValue();
                    if(!PeripheralData.ConveyorLineInputBowl && !ConveyorLineOutputBowl){
                        MessageLog.ShowUserMessage( UserLogEnum.订单处理日志 ,"机器人从" +item.FryingPanNum+"号位置取空盆到"+item.FryingPanNum + "号炒锅");
                        RobotModbus.getInstance().WriteShort("GI0",(short) (item.FryingPanNum+30),1,null);//写入机器人主任务,指定出餐空盆位置
                        RobotMealControl(item.FryingPanNum);//机器人出餐控制
                    }
                }
            }
            RemoveTask();//移除任务检测
        }
    }
}
