package com.bonait.bnframework.business;

import android.util.Log;

import com.bonait.bnframework.HBL.Task;
import com.bonait.bnframework.common.helper.I.IThread;
import com.bonait.bnframework.common.helper.ThreadManager;
import com.bonait.bnframework.common.modbus.ModbusMaster;
import com.bonait.bnframework.common.utils.ToastUtils;

public class YJDeviceHelper extends ModbusMaster {
    private  volatile  static  YJDeviceHelper _Instance;
    public synchronized  static YJDeviceHelper getInstance()
    {
        if(_Instance==null) _Instance=new YJDeviceHelper();
        return _Instance;
    }
    private YJDeviceHelper(){ }

    public void ReadData(){
        ThreadManager.Get().StartLong("外设PLC状态获取", true, new IThread() {
            @Override
            public void Run() throws Exception {
                ReadBool("M10.0", 6, 1, (bools)->{
                    PeripheralData.RawMaterialBowl=bools;
                });

                ReadBool("M10.6", 6, 1, (bools)->{
                    PeripheralData.SeasoningBasin=bools;
                });

                ReadBool("M11.4", 1, 1, (bools)->{
                    PeripheralData.EmptyBowl[0]=bools[0];
                });
                ReadBool("M11.5", 1, 1, (bools)->{
                    PeripheralData.EmptyBowl[1]=bools[0];
                });
                ReadBool("M12.0", 1, 1, (bools)->{//货架第三层出餐空盆
                    PeripheralData.EmptyBowl[2]=bools[0];
                });

                ReadBool("M12.2", 1, 1, (bools)->{//进料盆
                    PeripheralData.ConveyorLineInputBowl=bools[0];
                });

                ReadBool("M12.3", 1, 1, (bools)->{//出料盆
                    PeripheralData.ConveyorLineOutputBowl.setValue(bools[0]);
                });

                ReadBool("M15.0", 1, 1, (bools)->{//输送线状态
                    PeripheralData.ConveryorLineRuning.setValue(bools[0]);
                });

                ReadBool("M16.0", 1, 1, (bools)->{//输送电机过载
                    if(bools[0]){
                        ToastUtils.error("输送电机过载,请联系工作人员！");
                    }
                });

                ReadBool("M16.1", 1, 1, (bools)->{//输送电机通讯异常
                    if(bools[0]){
                        ToastUtils.error("输送电机通讯异常,请联系工作人员！");
                    }
                });


                Log.e("输送线检测","PeripheralData.ConveyorLineInputBowl="+PeripheralData.ConveyorLineInputBowl
                        +";PeripheralData.ConveyorLineOutputBowl="+PeripheralData.ConveyorLineOutputBowl.getValue());
                Log.e("空盆检测","11.4"+PeripheralData.EmptyBowl[0]+";11.5"+PeripheralData.EmptyBowl[1]+";12.0"+PeripheralData.EmptyBowl[2]);
                Thread.sleep(10);
            }

            @Override
            public void RunComplete() throws Exception {

            }
        });
    }

    /**
     * 主料位置验证
     * @param num
     * @return
     */
    public boolean MainMaterialLocVerify(int num){
        if(num>0&&num<=PeripheralData.RawMaterialBowl.length){
            return PeripheralData.RawMaterialBowl[num-1];
        }
        return false;
    }

    /**
     * 辅料位置验证
     * @param num
     * @return
     */
    public boolean AccessoriesLocVerify(int num){
        if(num>0&&num<=PeripheralData.SeasoningBasin.length){
            return PeripheralData.SeasoningBasin[num-1];
        }
        return false;
    }

    /**
     * 获取空盆位置
     * @return
     */
    public int getEmptyBlowLoc(){
        for (int i = 0; i <  PeripheralData.EmptyBowl.length; i++) {
            if( PeripheralData.EmptyBowl[i]) return i;
        }
        return -1;
    }

    public void ConveyorLineControl(boolean value){
        Task.Run(()->{
            WriteBool(value?"M0.0":"M0.1",true,1,null);
            Task.Delay(500);
            WriteBool(value?"M0.0":"M0.1",false,1,null);
        });
    }

    public void MainLampControl(int type){
        Task.Run(()->{
            switch (type){
                case 0:
                    WriteBool("M2.1",true,1,null);//绿
                    WriteBool("M2.0",false,1,null);//黄
                    WriteBool("M2.2",false,1,null);//红
                    WriteBool("M2.3",false,1,null);//蜂鸣器
                    break;
                case 1:
                    WriteBool("M2.1",false,1,null);//绿
                    WriteBool("M2.0",true,1,null);//黄
                    WriteBool("M2.2",false,1,null);//红
                    WriteBool("M2.3",false,1,null);//蜂鸣器
                    break;
                case 2:
                    WriteBool("M2.1",false,1,null);//绿
                    WriteBool("M2.0",false,1,null);//黄
                    WriteBool("M2.2",true,1,null);//红
                    WriteBool("M2.3",true,1,null);//蜂鸣器
                    break;
            }
        });
    }

    public void Close(){
        ThreadManager.Get().Stop("外设PLC状态获取");
        release();
    }

}
