package com.bonait.bnframework.business.deviceControl.seasonBar;

import android.os.Handler;
import android.os.Looper;

import com.bonait.bnframework.business.deviceControl.PLCType;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.helper.I.IReadCallBack;
import com.bonait.bnframework.common.helper.I.IWriteCallBack;
import com.bonait.bnframework.common.modbus.ModbusMaster;
import com.bonait.bnframework.common.utils.ToastUtils;

/**PLC控制类*/
public class PLCController{

    //region 单例模式
    private static volatile PLCController instance = null;
    public static PLCController get() {
        PLCController manager = instance;
        if (manager == null) {
            synchronized (PLCController.class) {
                manager = instance;
                if (manager == null) {
                    manager = new PLCController();
                    instance = manager;
                }
            }
        }
        return manager;
    }
    private PLCController(){};
    //endregion

    //region 私有变量
    /**设备的从站ID*/
    private final int slaveId=1;
    /**PLC类型是三菱的*/
    private final PLCType plcType=PLCType.MELSEC;
    //endregion


    public void Inital(IWriteCallBack callBack) {

        //TODO:确定初始化逻辑再写。
    }


    public <TDataType> TDataType readPLC(String address,Class<TDataType> dataType, IReadCallBack callBack) {
        if (Boolean.class.equals(dataType)) {

            return dataType.cast(true);
        } else if (Short.class.equals(dataType)) {


            return dataType.cast((short) 0);
        } else if (Integer.class.equals(dataType)) {


            return dataType.cast(0);
        } else if (Float.class.equals(dataType)) {

            return dataType.cast(0.0f);
        } else {
            // 其他类型的处理逻辑
            return null; // 默认返回 null
        }
    }


    public <TDataType> void writePLC(String address,TDataType value, IWriteCallBack callBack) {

//        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey(address)){
//            String addr=ConfigName.PLC_Address.get(address).address;
//            if (!addr.isEmpty()&&getModAddr(addr)!=-1){
//                try{
//                    String startAddr=String.valueOf(getModAddr(addr));
//                    //String startAddr=address;
//                    if (value instanceof Boolean){
//                        ModbusMaster.get().WriteBool(startAddr,(boolean)value,slaveId,callBack);
//                    } else if (value instanceof boolean[]) {
//                        ModbusMaster.get().WriteBool(startAddr,(boolean[])value,slaveId,callBack);
//                    } else if (value instanceof Integer) {
//                        ModbusMaster.get().WriteInt(startAddr,(int) value,slaveId,callBack);
//                    }else if (value instanceof int[]) {
//                        ModbusMaster.get().WriteInt(startAddr,(int[]) value,slaveId,callBack);
//                    }else if (value instanceof Short) {
//                        ModbusMaster.get().WriteShort(startAddr,(short) value,slaveId,callBack);
//                    }else if (value instanceof short[]) {
//                        ModbusMaster.get().WriteShort(startAddr,(short[])value,slaveId,callBack );
//                    }else if (value instanceof Float) {
//                        ModbusMaster.get().WriteFloat(startAddr,(float)value,slaveId,callBack );
//                    }else if (value instanceof Float[]) {
//                        ModbusMaster.get().WriteFloat(startAddr,(float[]) value,slaveId,callBack );
//                    }
//                }catch (Exception ex){
//                    ToastUtils.error("Write PLC error:"+ex.getMessage());
//                }
//
//            }
//        }
    }


    /**适用于普通仓
     * @param index 仓位编号。
     * @param delayTime 开启时间。
     * */
    public void openMotorAndClose(int index, int delayTime, IWriteCallBack callBack) {
        try {
            openMotor(index, new IWriteCallBack() {
                @Override
                public void onSuccess() {
                    new Handler(Looper.getMainLooper()).postDelayed(()->{
                        closeMotor(index, new IWriteCallBack() {
                            @Override
                            public void onSuccess() {
                                callBack.onSuccess();
                            }

                            @Override
                            public void onFailure(String ErrorMsg) {
                                callBack.onFailure(ErrorMsg);
                            }
                        });
                    },delayTime);
                }
                @Override
                public void onFailure(String ErrorMsg) {
                    callBack.onFailure(ErrorMsg);
                }
            });
        } catch (Exception ex) {
            ToastUtils.error("OpenMotorAndClose Exception:" + ex.getMessage());
        }
    }


    public void openMotor(int index, IWriteCallBack callBack) {

    }


    public void closeMotor(int index, IWriteCallBack callBack) {

    }


    /**适用于普通仓（打开后即可关闭，不需要反转）。
     * @param name 仓位名称
     * @param delayTime 打开时间
     * */
    public void openMotorAndClose(String name, int delayTime, IWriteCallBack callBack) {

    }

    /**打开电机并反转，最后关闭
     * @param name 仓位名称。
     * @param delayTime 开启时间。
     * @param reverseTime 反转时间。
     * */
    public void openAndReverse(String name,int delayTime,int reverseTime,IWriteCallBack callBack){

    }

    public void openMotor(String name, IWriteCallBack callBack) {
        writePLC(name,true,callBack);
    }
    public void reverseMotor(String name, IWriteCallBack callBack) {
        writePLC(name,false,callBack);
    }

    public void closeMotor(String name, IWriteCallBack callBack) {

    }


    public void clrHeartbeat(IWriteCallBack callBack) {
        writePLC("心跳时间",(short)0,callBack);
    }


    public boolean isTimeOut() {
        final boolean[] result = {true};
//        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey("断线标志")) {
//            String addr = ConfigName.PLC_Address.get("断线标志").address;
//            try {
//                int startAddr=getModAddr(addr);
//                if (!addr.isEmpty()&& startAddr!= -1) {
//                    ModbusMaster.get().ReadBool(String.valueOf(startAddr), 1, slaveId, val->{
//                        result[0]=val[0];
//                    });
//                }
//            } catch (Exception ex) {
//                ToastUtils.error("Exception message:" + ex.getMessage());
//            }
//        }
        return result[0];
    }


    /**适用于三菱PLC，只有X,Y,M,D区*/
    private int getModAddr(String address){
        if (address == null) return -1;
        address = address.toUpperCase().trim();

        try {
            if (address.length() < 2) return Integer.parseInt(address);

            String res = address.substring(1);
            if (res == null) return -1;

            int startAddress=Integer.parseInt(res) ;
            //如果是M区地址。
            if (address.contains("M")) {
                return startAddress + 4000;
            } else if (address.contains("D")) {
                return startAddress;
            }else if(address.contains("X")){
                return Integer.parseInt(res,8);
            }else if(address.contains("Y")){
                return Integer.parseInt(res,8)+300;
            }else{
                return Integer.parseInt(address);
            }
        } catch (Exception e) {
            return -1;
        }
    }
}
