package com.bonait.bnframework.common.ModbusSlave;

import android.util.Log;

import com.bonait.bnframework.common.helper.DataFormat;
import com.bonait.bnframework.common.modbus.ByteLib;
import com.bonait.bnframework.common.modbus.ShortLib;

import net.wimpi.modbus.ModbusCoupler;
import net.wimpi.modbus.net.ModbusTCPListener;
import net.wimpi.modbus.procimg.DigitalOut;
import net.wimpi.modbus.procimg.Register;
import net.wimpi.modbus.procimg.SimpleDigitalIn;
import net.wimpi.modbus.procimg.SimpleDigitalOut;
import net.wimpi.modbus.procimg.SimpleProcessImage;
import net.wimpi.modbus.procimg.SimpleRegister;

import java.net.Inet6Address;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class ModbusTcpSlave {
    SimpleProcessImage spi = new SimpleProcessImage();
    ModbusTCPListener listener = null;
    int port = 30000;//Android 1024 以下端口属于系统端口，需要root权限

    //参考连接
    //https://www.geek-share.com/detail/2805846670.html

    /**
     * 寄存器地址注册，地址依次从 add 添加顺序为主
     */
    public void Register(){

        //线圈寄存器（DO）
        for (int i = 0; i <400;i++)
            spi.addDigitalOut(new SimpleDigitalOut(false));

        //保持寄存器（AO）
        for (int i =0;i<300;i++)
            spi.addRegister(new SimpleRegister(0));

        //离散输入寄存器（DI）
//        spi.addDigitalIn(new SimpleDigitalIn(false));

//输入寄存器（AI）
//        spi.addInputRegister(new SimpleInputRegister(45));

    }

    /**
     * 寄存器注册
     */
    public void Init(){

        Register();

        ModbusCoupler.getReference().setProcessImage(spi);
        ModbusCoupler.getReference().setMaster(false);//默认是true，这里需要设置为false
        ModbusCoupler.getReference().setUnitID(1);//从站地址

        new Thread(networkTask).start();
        Log.e("TAG", "本机的IP = " + getHostIP());
    }


    /**
     * 网络操作相关的子线程
     */
    Runnable networkTask = new Runnable() {

        @Override
        public void run() {
            // TODO
            // 在这里进行 http request.网络请求相关操作

//            byte[] bytes =new byte[4];
//            bytes[0]=(byte)192;
//            bytes[1]=(byte)168;
//            bytes[2]=(byte)6;
//            bytes[3]=(byte)105;
//
//            InetAddress adder= null;
//            try {
//                adder = InetAddress.getByAddress(bytes);
//            } catch (UnknownHostException e) {
//                throw new RuntimeException(e);
//            }

            listener = new ModbusTCPListener(3 );
            listener.setPort(port);
            listener.start();
        }
    };

    public void WriteString(String str,int add){
        short[] shorts = ShortLib.ToShorts(str.getBytes(), DataFormat.ABCD);
        int tempAdd = add;
        Log.e("TAG", "写入字长度: "+shorts.length);
        for (int i = 0;i<shorts.length;i++)
        {
            Register register = spi.getRegister(tempAdd);
            register.setValue(shorts[i]);
            tempAdd++;
        }
    }

    public void WriteString(int add){
        short[] shorts = new short[18];
        int tempAdd = add;
        for (int i = 0;i<shorts.length;i++)
        {
            Register register = spi.getRegister(tempAdd);
            register.setValue(shorts[i]);
            tempAdd++;
        }
    }

    public void WriteBoolean(int add,boolean value){
        DigitalOut register= spi.getDigitalOut(add);
        register.set(value);
    }

    public boolean ReadBoolean(int add){
        DigitalOut register= spi.getDigitalOut(add);
       return register.isSet();
    }

    public String ReadString(int add){
        try{
            Register[]  register= spi.getRegisterRange(add,50);
            short[] data = new short[50];
            if(register!=null){
                for (int i=0;i<register.length;i++){
                    data[i]=register[i].toShort();
                }
            }
            byte[] bytes = ByteLib.ToBytes(data,DataFormat.ABCD);
            return new String(bytes, "UTF-8").trim();
        }catch (Exception e){}
        return "未知";
    }

    /**
     * 获取ip地址
     * @return
     */
    public  String getHostIP() {

        String hostIp = null;
        try {
            Enumeration nis = NetworkInterface.getNetworkInterfaces();
            InetAddress ia = null;
            while (nis.hasMoreElements()) {
                NetworkInterface ni = (NetworkInterface) nis.nextElement();
                Enumeration<InetAddress> ias = ni.getInetAddresses();
                while (ias.hasMoreElements()) {
                    ia = ias.nextElement();
                    if (ia instanceof Inet6Address) {
                        continue;// skip ipv6
                    }
                    String ip = ia.getHostAddress();
                    if (!"127.0.0.1".equals(ip)) {
                        hostIp = ia.getHostAddress();
                        break;
                    }
                }
            }
        } catch (SocketException e) {
            Log.i("yao", "SocketException");
            e.printStackTrace();
        }
        return hostIp;
    }
}


