package com.bonait.bnframework.common.db.util;

import android.content.ContentValues;
import android.database.sqlite.SQLiteDatabase;

import com.apkfuns.logutils.LogUtils;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.db.QueryDB;
import com.bonait.bnframework.common.db.file.DBHelper;
import com.bonait.bnframework.common.db.mode.BPA_CLOUDDATA;
import com.bonait.bnframework.common.db.mode.BPA_GOODS;
import com.bonait.bnframework.common.db.mode.BPA_GOODSRECIPE;
import com.bonait.bnframework.common.db.mode.BPA_MATERIAL;
import com.bonait.bnframework.common.db.mode.BPA_PROCESS;
import com.bonait.bnframework.common.db.mode.BPA_PROCESSModel;
import com.bonait.bnframework.common.helper.Tools;

import java.util.List;
import java.util.Map;

/**
 * @author: liup
 * @description: 批量增删
 * @date: 2024/7/23 9:48.
 */
public class DBListDataUtil {
    /**
     * 批量新增
     * @param list
     * @return
     */
    public static boolean addPROCESSModelList( List<BPA_PROCESSModel> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_PROCESSModel.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }

    public static boolean addPROCESSList( List<BPA_PROCESS> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_PROCESS.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }

    public static boolean addGOODSRECIPEList( List<BPA_GOODSRECIPE> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_GOODSRECIPE.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }

    public static boolean addMATERIALList( List<BPA_MATERIAL> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_MATERIAL.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }

    public static boolean addGOODSList( List<BPA_GOODS> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_GOODS.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }

    public static boolean addCLOUDDATAList( List<BPA_CLOUDDATA> list) {
        if(list==null || list.isEmpty()){
            return true;
        }
        QueryDB.lock.lock();
        SQLiteDatabase db = DBHelper.getInstance(ConfigName.getInstance().dishesCon).getWritableDatabase();
        try {
            long insert=-1;
            db.beginTransaction();
            for(int i = 0; i < list.size(); i++){
                ContentValues cv = new ContentValues();

                Map<String, Object> map = Tools.getObjValue(list.get(i));
                if (map.get("id").toString().isEmpty())
                    return false;
                for (String key : map.keySet()) {
                    Object value = map.get(key);
                    if (value instanceof String) {
                        cv.put(key, (String) value);
                    } else if (value instanceof Integer) {
                        cv.put(key, ((Integer) value).intValue());
                    } else if (value instanceof Double) {
                        cv.put(key, ((Double) value).doubleValue());
                    } else if (value instanceof Float) {
                        cv.put(key, ((Float) value).floatValue());
                    } else if (value instanceof Long) {
                        cv.put(key, ((Long) value).longValue());
                    } else if (value instanceof Boolean) {
                        cv.put(key, ((Boolean) value).booleanValue());
                    }
                }
                insert = db.insertOrThrow(BPA_CLOUDDATA.class.getSimpleName(), null, cv);
                if (insert == -1) {
                    throw new Exception("Failed to insert data at index " + i);
                }
            }
            LogUtils.d("removeList 批量添加工序");

            db.setTransactionSuccessful();
            return insert > 0;
        } catch (Exception e) {
            LogUtils.d("addList 批量新增异常");
        } finally {
            db.endTransaction();
            db.close();
            QueryDB.lock.unlock();
        }
        return false;
    }
}
