package com.bonait.bnframework.common.http.callback.json;

import android.content.Context;
import android.os.Build;
import android.view.View;
import android.view.Window;

import com.bonait.bnframework.common.helper.MessageLog;
import com.lzy.okgo.request.base.Request;
import com.qmuiteam.qmui.widget.dialog.QMUITipDialog;

/**
 * Created by LY on 2019/4/1.
 *
 * 在网络请求前显示一个loading，请求结束后取消loading
 *
 * 有加载框的网络请求回调
 *
 */
public abstract class JsonDialogCallback<T> extends JsonCallback<T> {

    private QMUITipDialog tipDialog;

    /**
     * 弹出加载框入口
     * */
    public JsonDialogCallback(Context context) {
        super();
        initDialog(context);
    }

    /**
     * 初始化加载框
     * */
    private void initDialog(Context context) {
        tipDialog = new QMUITipDialog.Builder(context)
                .setIconType(QMUITipDialog.Builder.ICON_TYPE_LOADING)
                .setTipWord("正在加载")
                .create();

        fullScreen( tipDialog.getWindow());
    }


    private void fullScreen(Window window){
        try{
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.JELLY_BEAN) {
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                    // 全屏显示，隐藏状态栏和导航栏，拉出状态栏和导航栏显示一会儿后消失。
                    // 启动游戏模式，设置状态栏和导航栏中的图标变小，变模糊或者弱化其效果
                    window.getDecorView().setSystemUiVisibility(
                            View.SYSTEM_UI_FLAG_LOW_PROFILE
                                    | View.SYSTEM_UI_FLAG_LAYOUT_STABLE
                                    | View.SYSTEM_UI_FLAG_LAYOUT_FULLSCREEN
                                    | View.SYSTEM_UI_FLAG_LAYOUT_HIDE_NAVIGATION
                                    | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                                    | View.SYSTEM_UI_FLAG_FULLSCREEN
                                    | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
                    //activity.getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION|View.SYSTEM_UI_FLAG_FULLSCREEN);
                    //activity.getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_FULLSCREEN | View.SYSTEM_UI_FLAG_HIDE_NAVIGATION);
                } else {
                    // 全屏显示，隐藏状态栏
                    window.getDecorView().setSystemUiVisibility(View.INVISIBLE);
                }
            }
        }catch ( Exception e){
            MessageLog.ShowError(e.getMessage());
        }
    }

    /**
     * 请求网络开始前弹出加载框
     * */
    @Override
    public void onStart(Request<T, ? extends Request> request) {
        if (tipDialog != null && !tipDialog.isShowing()) {
            tipDialog.show();
        }
    }

    /**
     * 请求网络结束后关闭加载框
     * */
    @Override
    public void onFinish() {
        if (tipDialog != null && tipDialog.isShowing()) {
            tipDialog.dismiss();
        }
    }
}
