package com.bonait.bnframework.common.image.utils;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Environment;
import android.util.Log;
import android.widget.ImageView;

import com.bonait.bnframework.MainApplication;
import com.bonait.bnframework.R;
import com.bonait.bnframework.common.utils.NetworkUtils;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

/**
 * 三级缓存之本地缓存
 */
public class LocalCacheUtils {

    private static final String CACHE_PATH= Environment.getExternalStorageDirectory().getAbsolutePath()+"/hblchayingdb/WebImage";

    //region 私有单例
    private static volatile LocalCacheUtils _instance;
    public static LocalCacheUtils Get() {
        if (_instance == null)
            _instance = new LocalCacheUtils();
        return _instance;
    }
    /**
     * 设置Image图片
     *
     * @param ivPic
     * @param name
     * @param url
     * @return
     */
    public boolean SetBitmapFile(ImageView ivPic, String name, String url) {
        if(name.contains(CACHE_PATH)){
            name = name.replace(CACHE_PATH,"");
        }
        File file = new File(CACHE_PATH, name);
        String imgUrl = null;
        if(url!=null && url.startsWith("http")){
            if(NetworkUtils.isNetworkConnected(MainApplication.getContext())){
                imgUrl = url;
            }
        }
        Log.d("LocalCacheUtils","SetBitmapFile imgUrl="+imgUrl+"  name="+name);
        if (null != file && file.exists()) {
            try {
                Log.d("LocalCacheUtils","加载图片");
                Glide.with(ivPic.getContext()).load(file)//imgUrl==null?file:imgUrl
                        //.override(width, height)
//                        .thumbnail(0.1f) // 加载原始图片的10%作为缩略图
                        .fitCenter() // 缩放图片以适应ImageView的尺寸
//                        .override(150, 75) // 设置目标图片的宽度和高度
                        .placeholder(R.mipmap.loading3)
//                        .error(R.mipmap.loadingerror)
                        .skipMemoryCache(true) // 禁用内存缓存
                        .diskCacheStrategy(DiskCacheStrategy.NONE)
                        .apply(new RequestOptions().transform(new FitXYRoundRadiusTransform(10)))
                        //.diskCacheStrategy(DiskCacheStrategy.ALL) // 缓存所有版本的图片
                        .into(ivPic);
                System.out.println("从本地获取图片啦.....");
                return true;
            } catch (OutOfMemoryError e) {
                e.printStackTrace();
            } catch (IllegalArgumentException ex) {
                System.out.println("界面已经销毁.....");
            }
        }
        return false;
    }

    /**
     * 从本地读取图片
     * @param url
     */
    public Bitmap getBitmapFromLocal(String url){
        String fileName = null;//把图片的url当做文件名,并进行MD5加密
        try {
            fileName = url;//MD5Encoder.encode(url);//AES.Encrypt(url);//MD5Encoder.encode(url);
            File file=new File(CACHE_PATH,fileName);
            Bitmap bitmap=null;
            if(file.exists())//
            {
                bitmap = BitmapFactory.decodeStream(new FileInputStream(file));
            }
            return bitmap;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }


    /**
     * 从网络获取图片后,保存至本地缓存
     *
     * @param url
     * @param bitmap
     */
    public String setBitmapToLocal(String url, Bitmap bitmap) {
        try {

            String fileName = url.substring(url.lastIndexOf('/') + 1);
            ;//MD5Encoder.encode(url);;//AES.Encrypt(url);//;MD5Encoder.encode(url);//把图片的url当做文件名,并进行MD5加密
            File file = new File(CACHE_PATH, fileName);
            //通过得到文件的父文件,判断父文件是否存在
            File parentFile = file.getParentFile();
            if (!parentFile.exists()) {
                parentFile.mkdirs();
            }

            if (!file.exists())//文件不存在
            {
                //把图片保存至本地
                bitmap.compress(Bitmap.CompressFormat.JPEG, 80, new FileOutputStream(file));
            }

            return fileName;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

}