package com.bonait.bnframework.common.modbus;


import static com.bonait.bnframework.common.modbus.DataConvertLib.BytesToFloat;
import static com.bonait.bnframework.common.modbus.DataConvertLib.BytesToInt;
import static com.bonait.bnframework.common.modbus.DataConvertLib.GetAddress;
import static com.bonait.bnframework.common.modbus.DataConvertLib.IntToShorts;

import com.bonait.bnframework.business.ConfigData;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.helper.DataFormat;
import com.bonait.bnframework.common.helper.I.IReadCallBack;
import com.bonait.bnframework.common.helper.I.IWriteCallBack;
import com.bonait.bnframework.common.helper.MessageLog;
import com.bonait.bnframework.common.utils.ToastUtils;
import com.licheedev.modbus4android.ModbusCallback;
import com.licheedev.modbus4android.ModbusParam;
import com.licheedev.modbus4android.ModbusRespException;
import com.licheedev.modbus4android.param.TcpParam;
import com.serotonin.modbus4j.ModbusMaster;
import com.serotonin.modbus4j.exception.ModbusInitException;
import com.serotonin.modbus4j.exception.ModbusTransportException;
import com.serotonin.modbus4j.msg.ReadCoilsResponse;
import com.serotonin.modbus4j.msg.ReadHoldingRegistersResponse;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.UnsupportedEncodingException;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.concurrent.ExecutionException;


public class ModbusClient {
    private static volatile ModbusClient instance = null;
    public static ModbusClient get() {
        ModbusClient manager = instance;
        if (manager == null) {
            synchronized (ModbusClient.class) {
                manager = instance;
                if (manager == null) {
                    manager = new ModbusClient();
                    instance = manager;
                }
            }
        }
        return manager;
    }
    private ModbusClient() {
    }

    static ModbusParam param;





    /**
     * Ping PLC地址是否通畅
     * @param address
     * @param pingTimes  ping的次数
     * @param timeOut  超时时间 10
     * @return
     */
//    public static boolean ping2(String address, int pingTimes, int timeOut) {
//        Process process = null;
//        try {
//            process = Runtime.getRuntime().exec( "ping "  + "-c " + pingTimes + " -w " + timeOut+ " "+address);
//            InputStreamReader r = new InputStreamReader(process.getInputStream());
//
//            LineNumberReader returnData = new LineNumberReader(r);
//
//            String returnMsg="";
//
//            String line = "";
//
//            while ((line = returnData.readLine()) != null) {
//
//                System.out.println(line);
//
//                returnMsg += line;
//
//            }
//
//            if(returnMsg.indexOf("100% packet loss")!=-1){
//                System.out.println("与 " +address +" 连接不畅通.");
//                //ToastUtils.info("与 " +address +" 连接不畅通.");
//                return  false;
//            } else{
//                System.out.println("与 " +address +" 连接畅通.");
//                //ToastUtils.info("与 " +address +" 连接不畅通.");
//                return true;
//            }
//        } catch (IOException e) {
//           // e.printStackTrace();
//        }
//        return false;
//
//    }
//
//
//    /**
//     * 连接
//     */
//    public void Connect() throws InterruptedException {
//        boolean status = false;
//        while (!status) {
//            try {
//                //status为0则代表通，为1则代表不通。
//                status =ping2(ConfigName.getInstance().Address,1,1);
//                Thread.sleep(1000);
//            } catch (InterruptedException e) {
//                MessageLog.ShowInfo("设备 " + ConfigName.getInstance().Address + " 网络验证失败");
//            } catch (Exception e) {
//                MessageLog.ShowInfo("设备 " + ConfigName.getInstance().Address + " 网络验证失败");
//            }
//        }
//        MessageLog.ShowInfo("设备 " + ConfigName.getInstance().Address + " PLC通讯正常,准备连接！");
//
//        while (ConfigName.getInstance().PlcIsConnect==false)
//        {
//            ConnectPLC();
//            Thread.sleep(5000);
//        }
//    }
//
//    /**
//     * 连接PLC
//     */
//    public static void ConnectPLC()
//    {
//        String host=ConfigName.getInstance().Address;
//        int port=ConfigName.getInstance().Post;
//        param = TcpParam.create(host, port)
//                .setTimeout(1000)
//                .setRetries(0)
//                .setEncapsulated(false)
//                .setKeepAlive(true);
//        ModbusBasis.get().init(param, new ModbusCallback<ModbusMaster>() {
//            @Override
//            public void onSuccess(ModbusMaster modbusMaster) {
//                ToastUtils.info("设备 " + ConfigName.getInstance().Address + " 连接成功");
//
//                MessageLog.ShowInfo("设备 " + ConfigName.getInstance().Address + " 连接成功");
//                ConfigName.getInstance().PlcIsConnect = true;
//                ConfigData.getInstance().PLC_Init();
//            }
//
//            @Override
//            public void onFailure(Throwable tr) {
//                ConfigName.getInstance().PlcIsConnect = false;
//                ToastUtils.info("设备 " + ConfigName.getInstance().Address + " 连接失败：" + tr.getMessage());
//                MessageLog.ShowError("设备 " + ConfigName.getInstance().Address + " 连接失败：" + tr.getMessage());
//            }
//            @Override
//            public void onFinally() {
//
//            }
//        });
//    }
//
//    /***
//     *读取实时状态
//     * @param Address the address
//     * @param length 读取的长度 3
//     * @param callback 读取成功的回调
//     */
//    public void ReadStatus(String Address, int length, IReadCallBack<byte[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length);
//            byte[] data = res.getData();
//            byte[] tempData = new byte[8];
//            tempData[0] = data[1];
//            tempData[1] = data[0];
//            tempData[2] = data[3];
//            tempData[3] = data[2];
//            tempData[4] = data[5];
//            tempData[5] = data[4];
//            tempData[6] = data[7];
//            tempData[7] = data[6];
//            if (callback != null) callback.onSuccess(tempData);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//    /**
//     * 读取补偿算法
//     * @param Address
//     * @param length
//     * @param callback
//     */
//    public void ReadbcStatus(String Address, int length, IReadCallBack<byte[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//       new ExceptionServer(Address, length, new IExceptionHandling() {
//           @Override
//           public void HandleException() throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
//               ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length);
//               byte[] data = res.getData();
//               byte[] tempData = new byte[1];
//               tempData[0] = data[1];
//               if (callback != null) callback.onSuccess(tempData);
//           }
//       });
//    }
//
//
//    /***
//     *读取扫码数据
//     * @param Address the address
//     * @param length 读取的长度
//     * @param callback 读取成功的回调
//     */
//    public void ReadString(String Address, int length, IReadCallBack<String> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length);
//            byte[] data = res.getData();
//            byte[] data1 = Arrays.copyOfRange(data, 0, data.length);
//            //36 GUID 18 Number
//            String id = "";
//            try {
//                byte[] tempdata = new byte[data.length];
//                int index=0;
//                for (int i = 0; i < data1.length; i++) {
//                    if (i % 2 == 0) {
//
//                        tempdata[i + 1] = data1[i];
//                    } else {
//                        tempdata[i - 1] = data1[i];
//                    }
//
//                    if (data1[i]==0 && index==0)
//                    {
//                        index=i;
//                    }
//                }
//
//                id = new String(tempdata, "UTF-8").trim();
//            } catch (UnsupportedEncodingException ex) {
//            }
//            if (callback != null) callback.onSuccess(id);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//
//    public void WriteString(String Address, int length, IWriteCallBack callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        short[] send = new short[length];
//        for (short item : send) {
//            item = 0;
//        }
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//            if (callback != null)
//                callback.onSuccess();
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        }
//    }
//
//    public void ReadShort(String Address, int length, IReadCallBack<short[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length);
//            short[] data = res.getShortData();
//            if (data.length == length) {
//                if (callback != null) callback.onSuccess(data);
//            }
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadShort onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//
//    public void ReadBool(String Address, int length, IReadCallBack<boolean[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadCoilsResponse res = ModbusBasis.get().syncReadCoil(1, add, length);
//            boolean[] data = res.getBooleanData();
//            boolean[] result = Arrays.copyOfRange(data, 0, length);
//            if (result.length == length) {
//                if (callback != null) callback.onSuccess(result);
//            }
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadBool onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadBool onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadBool onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadBool onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadBool onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//
//    public void ReadFloat(String Address, int length, IReadCallBack<float[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length*2);
//            byte[] data = res.getData();
//            float[] tempValues = new float[length];
//            for (int i = 0; i < length; i++) {
//                byte[] tempData = new byte[4];
//                for (int m = 0; m < 4; m++) {
//                    tempData[m] = data[i * 4 + m];
//                }
//                tempValues[i] = BytesToFloat(tempData, DataFormat.BADC);
//            }
//            if (tempValues.length == length) {
//                if (callback != null) callback.onSuccess(tempValues);
//            }
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadFloat onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadFloat onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadFloat onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadFloat onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadFloat onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//
//    public void ReadInt(String Address, int length, IReadCallBack<int[]> callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ReadHoldingRegistersResponse res = ModbusBasis.get().syncReadHoldingRegisters(1, add, length * 2);
//            byte[] data = res.getData();
//            int[] tempValues = new int[length];
//            if(data.length>=4)
//            {
//                for (int i = 0; i < length; i++) {
//                    byte[] tempData = new byte[4];
//                    for (int m = 0; m < 4; m++) {
//                        tempData[m] = data[i * 4 + m];
//                    }
////                Integer a1 = tempValues[i] = BytesToInt(tempData, DataFormat.ABCD);
////                Integer a2 = tempValues[i] = BytesToInt(tempData, DataFormat.BADC);
////                Integer a3 = tempValues[i] = BytesToInt(tempData, DataFormat.CDAB);
////                Integer a4 = tempValues[i] = BytesToInt(tempData, DataFormat.DCBA);
//                    tempValues[i] = BytesToInt(tempData, DataFormat.BADC);
//                }
//            }else
//            {
//                MessageLog.ShowError("ReadInt onFailure，Address=" );
//
//            }
//            if (callback != null) callback.onSuccess(tempValues);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("ReadInt onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("ReadInt onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("ReadInt onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("ReadInt onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("ReadInt onFailure，Address=" + Address + ",length=" + length + ",msg:" + e.toString());
//        }
//    }
//
//    public void WriteShort(String Address, short Value, IWriteCallBack callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        short[] send = new short[1];
//        send[0] = Value;
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//            if (callback != null)
//                callback.onSuccess();
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        }
//    }
//
//    public void WriteBool(String Address, boolean Value, IWriteCallBack callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        try {
//            ModbusBasis.get().syncWriteCoil(1, add, Value);
//            if (callback != null)
//                callback.onSuccess();
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        }
//    }
//
//    public void WriteFloat(String Address, float Value, IWriteCallBack callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        int intBits = Float.floatToRawIntBits(Value);
//        short[] send = IntToShorts(intBits);
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//
//            if (callback != null) callback.onSuccess();
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        }
//    }
//
//    public void WriteInt(String Address, int Value, IWriteCallBack callback) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        short[] send = IntToShorts(Value);
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//            if (callback != null) callback.onSuccess();
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//            if (callback != null) callback.onFailure(e.toString());
//        }
//    }
//
//    public void WriteShort(String Address, short Value) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        short[] send = new short[1];
//        send[0] = Value;
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteShort onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        }
//    }
//
//    public void WriteBool(String Address, boolean Value) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//
//        try {
//            ModbusBasis.get().syncWriteCoil(1, add, Value);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteBool onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        }
//    }
//
//    public void WriteFloat(String Address, float Value) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        int intBits = Float.floatToRawIntBits(Value);
//        short[] send = new short[]{(short) ((intBits >> 16) & 0xffff), (short) (intBits & 0xffff)};
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteFloat onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        }
//    }

//    public void WriteInt(String Address, int Value) {
//        int add = GetAddress(Address);
//        if (add < 0) return;
//        short[] send = IntToShorts(Value);
//        try {
//            ModbusBasis.get().syncWriteRegisters(1, add, send);
//        } catch (InterruptedException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ExecutionException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusTransportException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusInitException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        } catch (ModbusRespException e) {
//            MessageLog.ShowError("WriteInt onFailure，Address=" + Address + ",length=" + Value + ",msg:" + e.toString());
//        }
//    }

}
