package com.bonait.bnframework.common.modbus;

import static com.bonait.bnframework.common.modbus.DataConvertLib.ByteToShort;
import static com.bonait.bnframework.common.modbus.DataConvertLib.BytesToFloat;
import static com.bonait.bnframework.common.modbus.DataConvertLib.BytesToInt;
import static com.bonait.bnframework.common.modbus.DataConvertLib.IntToShorts;

import com.bonait.bnframework.HBL.NotifyPropVar;
import com.bonait.bnframework.common.helper.DataFormat;
import com.bonait.bnframework.common.helper.I.IReadCallBack;
import com.bonait.bnframework.common.helper.I.IWriteCallBack;
import com.bonait.bnframework.common.helper.MessageLog;
import com.licheedev.modbus4android.ModbusCallback;
import com.licheedev.modbus4android.ModbusParam;
import com.licheedev.modbus4android.ModbusRespException;
import com.licheedev.modbus4android.ModbusWorker;
import com.licheedev.modbus4android.param.SerialParam;
import com.licheedev.modbus4android.param.TcpParam;
import com.serotonin.modbus4j.exception.ModbusInitException;
import com.serotonin.modbus4j.exception.ModbusTransportException;
import com.serotonin.modbus4j.msg.ReadCoilsResponse;
import com.serotonin.modbus4j.msg.ReadHoldingRegistersResponse;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.UnsupportedEncodingException;
import java.util.Arrays;
import java.util.concurrent.ExecutionException;

public class ModbusMaster implements IRead,IWrite,IModbusMaster {
    //https://blog.csdn.net/zgkxzx/article/details/71630301
//    ModbusFactory factory = new ModbusFactory();//使用该方法可以创建不同类型的modbus通讯

    private ModbusWorker mw = new ModbusWorker();

    public void release(){
        try{
            mw.release();
        }catch (Exception e){}
    }

    /**
     * Ping PLC地址是否通畅
     * @param address
     * @param pingTimes  ping的次数
     * @param timeOut  超时时间 10
     * @return
     */
    private  boolean ping2(String address, int pingTimes, int timeOut) {
        Process process = null;
        try {
            process = Runtime.getRuntime().exec( "ping "  + "-c " + pingTimes + " -w " + timeOut+ " "+address);
            InputStreamReader r = new InputStreamReader(process.getInputStream());
            LineNumberReader returnData = new LineNumberReader(r);
            String returnMsg="";
            String line = "";
            while ((line = returnData.readLine()) != null) {
                System.out.println(line);
                returnMsg += line;
            }

            if(returnMsg.indexOf("100% packet loss")!=-1){
                System.out.println("与 " +address +" 连接不畅通.");
                //ToastUtils.info("与 " +address +" 连接不畅通.");
                return  false;
            } else{
                System.out.println("与 " +address +" 连接畅通.");
                //ToastUtils.info("与 " +address +" 连接不畅通.");
                return true;
            }
        } catch (IOException e) {
            // e.printStackTrace();
        }
        return false;

    }
    public boolean IsConnected=false;
    public String TempIp;
    private int TempPort;
    private String TempCom;
    private int TempBaudRate;

    /**
     * 连接类型，1=TCP,2=RUT
     */
    private int ConnectionType;

    private void ReConnect(String error){
        if(ConnectionType==1){
            if(error.contains("Connection refused")||error.contains("SocketException")||error.contains("SocketTimeoutException")){
                IsConnected=false;
                ConnectStatus.setValue(false);
                TcpConnect(TempIp,TempPort,null);
            }else{
                MessageLog.ShowError("Tcp 重连失败："+error);
            }
        }
        else if (ConnectionType==2){
            if(error.contains("Connection refused")){
                IsConnected=false;
                ConnectStatus.setValue(false);
                RtuConnect(TempCom,TempBaudRate,null);
            }else{
                MessageLog.ShowError("Rtu 重连失败："+error);
            }
        }

    }

    boolean IsOutErrorLog=false;

//    public IRun ConnectedNotify;

    /**
     * 连接状态通知
     */
    public NotifyPropVar<Boolean> ConnectStatus=new NotifyPropVar<>(false);

    /**
     * TCP 连接，连接时请用多线程的方式，否者有可能无法连接
     * @param ip
     * @param port
     * @param result
     * @throws InterruptedException
     */
    @Override
    public void TcpConnect(String ip, int port, IOperateResult result){
        TempIp=ip;
        TempPort=port;
        ConnectionType=1;
        MessageLog.ShowInfo("开始验证网络："+ip );
        boolean status = false;
        while (!status) {
            try {
                //status为0则代表通，为1则代表不通。
                status =ping2(ip,1,1);
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                MessageLog.ShowInfo("设备 " + ip + " 网络验证失败");
            } catch (Exception e) {
                MessageLog.ShowInfo("设备 " + ip + " 网络验证失败");
            }
        }
        MessageLog.ShowInfo("设备 " + ip + " 网络验证成功");
        ModbusParam param= TcpParam.create(ip, port)
                .setTimeout(1000)
                .setRetries(0)
                .setEncapsulated(false)
                .setKeepAlive(true);
        IsOutErrorLog=false;
//       final AtomicReference<Boolean> IsOutErrorLog = new AtomicReference<>();
        while (IsConnected==false)
        {
            mw.init(param, new ModbusCallback<com.serotonin.modbus4j.ModbusMaster>() {
                @Override
                public void onSuccess(com.serotonin.modbus4j.ModbusMaster modbusMaster) {
                    IsConnected=true;
                    if(result!=null) result.onSuccess();
//                    if(ConnectedNotify!=null)ConnectedNotify.Run();
                    ConnectStatus.setValue(true);
                    MessageLog.ShowInfo("Modbus TCP 连接成功" );
                }
                @Override
                public void onFailure(Throwable tr) {
                    IsConnected = false;
                    ConnectStatus.setValue(false);
                    if(result!=null) result.onFailure(tr.getMessage());
                  if(!IsOutErrorLog)  {
                      MessageLog.ShowInfo("Modbus TCP 连接失败" );
//                      IsOutErrorLog.set(true);
                      IsOutErrorLog=true;
                  }
                }
                @Override
                public void onFinally() {
                    if(result!=null) result.onFinally();
                }
            });
            try
            {
                if(IsConnected==false) Thread.sleep(5000);
            }catch(InterruptedException ex){

            }
        }
    }

    /**
     * RTU 串口连接，建议连接时用多线程的方式，否者有可能无法连接
     * @param com
     * @param baudRate
     * @param result
     * @throws InterruptedException
     */
    @Override
    public void RtuConnect(String com, int baudRate, IOperateResult result){
        MessageLog.ShowInfo("开始连接Modbus RTU" );
        TempCom=com;
        TempBaudRate=baudRate;
        ConnectionType=2;
        ModbusParam param=SerialParam.create(com, baudRate) // 串口地址和波特率
                .setDataBits(8) // 数据位
                .setParity(0) // 校验位
                .setStopBits(1) // 停止位
                .setTimeout(500).setRetries(0); // 不重试
        while (IsConnected==false)
        {
            mw.init(param, new ModbusCallback<com.serotonin.modbus4j.ModbusMaster>() {
                @Override
                public void onSuccess(com.serotonin.modbus4j.ModbusMaster modbusMaster) {
                    IsConnected=true;
                    ConnectStatus.setValue(true);
                    if(result!=null) result.onSuccess();
                    MessageLog.ShowInfo("Modbus RTU 连接成功" );
                }
                @Override
                public void onFailure(Throwable tr) {
                    IsConnected = false;
                    ConnectStatus.setValue(false);
                    if(result!=null) result.onFailure(tr.getMessage());
                    MessageLog.ShowInfo("Modbus RTU 连接失败" );
                }
                @Override
                public void onFinally() {
                    if(result!=null) result.onFinally();
                }
            });
            try{
                if(IsConnected==false) Thread.sleep(5000);
            }catch (InterruptedException ex){

            }
        }
    }

    @Override
    public void ReadBool(String address, int length, int slaveId, IReadCallBack<boolean[]> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                ReadCoilsResponse res = mw.syncReadCoil(slaveId, add, length);
                boolean[] data = res.getBooleanData();
                boolean[] result = Arrays.copyOfRange(data, 0, length);
                if (callback != null) callback.onSuccess(result);
            }

            @Override
            public void onFailure(String error)  {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadShort(String address, int length,int slaveId, IReadCallBack<short[]> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                ReadHoldingRegistersResponse res = mw.syncReadHoldingRegisters(slaveId, add, length);
                short[] data = res.getShortData();
                if (data.length == length) {
                    if (callback != null) callback.onSuccess(data);
                }
            }

            @Override
            public void onFailure(String error)  {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadInt(String address, int length, int slaveId, DataFormat df, IReadCallBack<int[]> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                ReadHoldingRegistersResponse res = mw.syncReadHoldingRegisters(slaveId, add, length * 2);
                byte[] data = res.getData();
                int[] tempValues = new int[length];
                if(data.length==4*length)
                {
                    for (int i = 0; i < length; i++) {
                        byte[] tempData = new byte[4];
                        for (int m = 0; m < 4; m++) {
                            tempData[m] = data[i * 4 + m];
                        }
                        tempValues[i] = BytesToInt(tempData, df);
                    }
                   if (callback != null) callback.onSuccess(tempValues);
                }else{
                    MessageLog.ShowError("ReadInt onFailure，数据长度不匹配,读取长度："+length+"返回长度："+data.length );
                }
            }

            @Override
            public void onFailure(String error)  {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadInt(String address, int length, int slaveId, IReadCallBack<int[]> callback) {
        if (!IsConnected) {
            return;
        }
        ReadInt(address, length, slaveId,DataFormat.BADC, callback);
    }

    @Override
    public void ReadByte(String address, int length, int slaveId, IReadCallBack<byte[]> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
               int tempLength=length;
                if (length%2!=0){
                    tempLength++;
               }
                ReadHoldingRegistersResponse res = mw.syncReadHoldingRegisters(slaveId, add, tempLength );
                byte[] data = res.getData();
                if(data.length==tempLength)
                {
                    byte[] result = Arrays.copyOfRange(data, 0, length);
                    if (callback != null) callback.onSuccess(result);
                }else
                {
                    MessageLog.ShowError("ReadInt onFailure，数据长度不匹配,读取长度："+length+"返回长度："+data.length );
                }
            }

            @Override
            public void onFailure(String error) {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadString(String address, int length,  int slaveId, IReadCallBack<String> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                ReadHoldingRegistersResponse res = mw.syncReadHoldingRegisters(slaveId, add, length);
                byte[] data = res.getData();
                try {
                    if (data.length ==length*2) {
                        if (callback != null) callback.onSuccess(new String(data, "UTF-8").trim());
                    }
                } catch (UnsupportedEncodingException ex) {
                }
            }

            @Override
            public void onFailure(String error)  {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadFloat(String address, int length, int slaveId, DataFormat df, IReadCallBack<float[]> callback) {
        if (!IsConnected) {
            return;
        }
        new ExceptionServer(address, length, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                ReadHoldingRegistersResponse res = mw.syncReadHoldingRegisters(slaveId, add, length*2);
                byte[] data = res.getData();
                float[] tempValues = new float[length];
                for (int i = 0; i < length; i++) {
                    byte[] tempData = new byte[4];
                    for (int m = 0; m < 4; m++) {
                        tempData[m] = data[i * 4 + m];
                    }
                    tempValues[i] = BytesToFloat(tempData, df);
                }
                if (tempValues.length == length) {
                    if (callback != null) callback.onSuccess(tempValues);
                }
            }

            @Override
            public void onFailure(String error)  {
                ReConnect(error);
            }
        });
    }

    @Override
    public void ReadFloat(String address, int length, int slaveId, IReadCallBack<float[]> callback) {
        if (!IsConnected) {
            return;
        }
        ReadFloat(address,length,slaveId,DataFormat.BADC,callback);
    }




    @Override
    public void WriteBool(String address, boolean value, int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                mw.syncWriteCoil(slaveId, add, value);
                if (callback != null)callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteBool(String address, boolean[] value,  int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                mw.syncWriteCoils(slaveId, add, value);
                if (callback != null)callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteShort(String address, short value,   int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                short[] send = new short[1];
                send[0] = value;
                mw.syncWriteRegisters(slaveId, add, send);
                if (callback != null)callback.onSuccess();
            }

            @Override
            public void onFailure(String error) {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteShort(String address, short[] value,   int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                mw.syncWriteRegisters(slaveId, add, value);
                if (callback != null)callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteInt(String address, int value,  int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                short[] send = IntToShorts(value);
                mw.syncWriteRegisters(slaveId, add, send);
                if (callback != null) callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteInt(String address, int[] value,  int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
              short[] send=new short[value.length*2];
               for(int i=0;i<value.length;i=i+2)
               {
                   short[] temp = IntToShorts(value[i]);
                  if (temp.length==2)
                  {
                      send[i]=temp[0];
                      send[i+1]=temp[1];
                  }
               }
                mw.syncWriteRegisters(slaveId, add,send);
                if (callback != null) callback.onSuccess();
            }

            @Override
            public void onFailure(String error) {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteString(String address, String value,   int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
               byte[] bytes= value.getBytes();
                short[] send =ByteToShort(bytes);
                mw.syncWriteRegisters(slaveId, add, send);
                if (callback != null) callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteFloat(String address, float value, int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                int intBits = Float.floatToRawIntBits(value);
                short[] send = IntToShorts(intBits);
                mw.syncWriteRegisters(slaveId, add, send);
                if (callback != null) callback.onSuccess();
            }

            @Override
            public void onFailure(String error)  {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }

    @Override
    public void WriteFloat(String address, float[] value,  int slaveId, IWriteCallBack callback) {
        if (!IsConnected) {
            if (callback != null) callback.onFailure("设备未连接");
            return;
        }
        new ExceptionServer(address, 1, new IExceptionHandling() {
            @Override
            public void onSuccess(int add) throws InterruptedException, ExecutionException, ModbusTransportException, ModbusInitException, ModbusRespException {
                short[] send=new short[value.length*2];
                for(int i=0;i<value.length;i=i+2)
                {
                    int intBits = Float.floatToRawIntBits(value[i]);
                    short[] temp = IntToShorts(intBits);
                    if (temp.length==2)
                    {
                        send[i]=temp[0];
                        send[i+1]=temp[1];
                    }
                }
                mw.syncWriteRegisters(slaveId, add, send);
                if (callback != null) callback.onSuccess();
            }

            @Override
            public void onFailure(String error) {
                if (callback != null)callback.onFailure(error);
                ReConnect(error);
            }
        });
    }


}
